@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2008 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,10 +73,13 @@ struct GSWindowPrivate
         char      *away_message;
 
         GtkWidget *vbox;
+        GtkWidget *drawing_area;
         GtkWidget *lock_box;
         GtkWidget *lock_socket;
         GtkWidget *keyboard_socket;
 
+        GdkPixmap *background_pixmap;
+
         guint      popup_dialog_idle_id;
 
         guint      dialog_map_signal_id;
@@ -202,12 +205,12 @@ force_no_pixmap_background (GtkWidget *widget)
                                      "      bg[ACTIVE] = { 0.0, 0.0, 0.0 }\n"
                                      "      bg[PRELIGHT] = { 0.0, 0.0, 0.0 }\n"
                                      "   }\n"
-                                     "   widget \"gs-window*\" style : highest \"gs-theme-engine-style\"\n"
+                                     "   widget \"gs-window-drawing-area*\" style : highest \"gs-theme-engine-style\"\n"
                                      "\n");
                 first_time = FALSE;
         }
 
-        gtk_widget_set_name (widget, "gs-window");
+        gtk_widget_set_name (widget, "gs-window-drawing-area");
 }
 
 static void
@@ -244,7 +247,7 @@ clear_children (Window window)
 }
 
 static void
-clear_all_children (GSWindow *window)
+widget_clear_all_children (GtkWidget *widget)
 {
         GdkWindow *w;
 
@@ -252,20 +255,37 @@ clear_all_children (GSWindow *window)
 
         gdk_error_trap_push ();
 
-        w = GTK_WIDGET (window)->window;
+        w = widget->window;
 
         clear_children (GDK_WINDOW_XID (w));
 
-        gdk_display_sync (gtk_widget_get_display (GTK_WIDGET (window)));
+        gdk_display_sync (gtk_widget_get_display (widget));
         gdk_error_trap_pop ();
 }
 
 void
-gs_window_clear (GSWindow *window)
+gs_window_set_background_pixmap (GSWindow  *window,
+                                 GdkPixmap *pixmap)
+{
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        if (window->priv->background_pixmap != NULL) {
+                g_object_unref (window->priv->background_pixmap);
+        }
+
+        if (pixmap != NULL) {
+                window->priv->background_pixmap = g_object_ref (pixmap);
+                gdk_window_set_back_pixmap (GTK_WIDGET (window)->window,
+                                            pixmap,
+                                            FALSE);
+        }
+}
+
+static void
+gs_window_clear_to_background_pixmap (GSWindow *window)
 {
-        GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
-        GdkColormap *colormap;
         GtkStateType state;
+        GtkStyle    *style;
 
         g_return_if_fail (GS_IS_WINDOW (window));
 
@@ -273,29 +293,101 @@ gs_window_clear (GSWindow *window)
                 return;
         }
 
-        gs_debug ("Clearing window");
+        if (window->priv->background_pixmap == NULL) {
+                /* don't allow null pixmaps */
+                return;
+        }
+
+        gs_debug ("Clearing window to background pixmap");
+
+        style = gtk_style_copy (GTK_WIDGET (window)->style);
 
         state = (GtkStateType) 0;
-        while (state < (GtkStateType) G_N_ELEMENTS (GTK_WIDGET (window)->style->bg)) {
-                gtk_widget_modify_bg (GTK_WIDGET (window), state, &color);
+        while (state < (GtkStateType) G_N_ELEMENTS (GTK_WIDGET (window)->style->bg_pixmap)) {
+
+                if (style->bg_pixmap[state] != NULL) {
+                        g_object_unref (style->bg_pixmap[state]);
+                }
+
+                style->bg_pixmap[state] = g_object_ref (window->priv->background_pixmap);
                 state++;
         }
 
-        colormap = gdk_drawable_get_colormap (GTK_WIDGET (window)->window);
-        gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
-        gdk_window_set_background (GTK_WIDGET (window)->window, &color);
+        gtk_widget_set_style (GTK_WIDGET (window), style);
+        g_object_unref (style);
+
+        if (window->priv->background_pixmap != NULL) {
+                gdk_window_set_back_pixmap (GTK_WIDGET (window)->window,
+                                            window->priv->background_pixmap,
+                                            FALSE);
+        }
+
         gdk_window_clear (GTK_WIDGET (window)->window);
 
+        gdk_flush ();
+}
+
+static void
+clear_widget (GtkWidget *widget)
+{
+        GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
+        GdkColormap *colormap;
+        GtkStateType state;
+        GtkStyle    *style;
+
+        if (! GTK_WIDGET_VISIBLE (widget)) {
+                return;
+        }
+
+        gs_debug ("Clearing widget");
+
+        state = (GtkStateType) 0;
+        while (state < (GtkStateType) G_N_ELEMENTS (widget->style->bg)) {
+                gtk_widget_modify_bg (widget, state, &color);
+                state++;
+        }
+
+        style = gtk_style_copy (widget->style);
+
+        state = (GtkStateType) 0;
+        while (state < (GtkStateType) G_N_ELEMENTS (widget->style->bg_pixmap)) {
+
+                if (style->bg_pixmap[state] != NULL) {
+                        g_object_unref (style->bg_pixmap[state]);
+                }
+
+                style->bg_pixmap[state] = NULL;
+                state++;
+        }
+
+        colormap = gdk_drawable_get_colormap (widget->window);
+        gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
+        gdk_window_set_background (widget->window, &color);
+
+        gtk_widget_set_style (widget, style);
+        g_object_unref (style);
+
+        gdk_window_clear (widget->window);
+
         /* If a screensaver theme adds child windows we need to clear them too */
-        clear_all_children (window);
+        widget_clear_all_children (widget);
 
         gdk_flush ();
 }
 
+void
+gs_window_clear (GSWindow *window)
+{
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        clear_widget (GTK_WIDGET (window));
+        clear_widget (window->priv->drawing_area);
+}
+
 static GdkRegion *
 get_outside_region (GSWindow *window)
 {
-        int i;
+        int        i;
         GdkRegion *region;
 
         region = gdk_region_new ();
@@ -314,7 +406,8 @@ static void
 update_geometry (GSWindow *window)
 {
         GdkRectangle geometry;
-        GdkRegion *outside_region, *monitor_region;
+        GdkRegion   *outside_region;
+        GdkRegion   *monitor_region;
 
         outside_region = get_outside_region (window);
 
@@ -323,10 +416,10 @@ update_geometry (GSWindow *window)
                                          &geometry);
         monitor_region = gdk_region_rectangle (&geometry);
         gdk_region_subtract (monitor_region, outside_region);
-        gdk_region_destroy (outside_region); 
+        gdk_region_destroy (outside_region);
 
         gdk_region_get_clipbox (monitor_region, &geometry);
-        gdk_region_destroy (monitor_region); 
+        gdk_region_destroy (monitor_region);
 
         window->priv->geometry.x = geometry.x;
         window->priv->geometry.y = geometry.y;
@@ -338,6 +431,7 @@ static void
 screen_size_changed (GdkScreen *screen,
                      GSWindow  *window)
 {
+        gs_debug ("Got screen size changed signal");
         gtk_widget_queue_resize (GTK_WIDGET (window));
 }
 
@@ -353,6 +447,13 @@ gs_window_move_resize_window (GSWindow *window,
 
         g_assert (GTK_WIDGET_REALIZED (widget));
 
+        gs_debug ("Move and/or resize window on monitor %d: x=%d y=%d w=%d h=%d",
+                  window->priv->monitor,
+                  window->priv->geometry.x,
+                  window->priv->geometry.y,
+                  window->priv->geometry.width,
+                  window->priv->geometry.height);
+
         if (move && resize) {
                 gdk_window_move_resize (widget->window,
                                         window->priv->geometry.x,
@@ -822,6 +923,14 @@ gs_window_get_gdk_window (GSWindow *window)
         return GTK_WIDGET (window)->window;
 }
 
+GtkWidget *
+gs_window_get_drawing_area (GSWindow *window)
+{
+        g_return_val_if_fail (GS_IS_WINDOW (window), NULL);
+
+        return window->priv->drawing_area;
+}
+
 /* just for debugging */
 static gboolean
 error_watch (GIOChannel   *source,
@@ -882,7 +991,6 @@ spawn_on_window (GSWindow *window,
         int         standard_error;
         int         child_pid;
         int         id;
-        int         i;
 
         error = NULL;
         if (! g_shell_parse_argv (command, &argc, &argv, &error)) {
@@ -1360,6 +1468,8 @@ lock_command_watch (GIOChannel   *source,
                         add_emit_deactivated_idle (window);
                 }
 
+                gtk_widget_show (window->priv->drawing_area);
+
                 gs_window_clear (window);
                 set_invisible_cursor (GTK_WIDGET (window)->window, TRUE);
                 g_signal_emit (window, signals [DIALOG_DOWN], 0);
@@ -1401,7 +1511,7 @@ is_logout_enabled (GSWindow *window)
 static gboolean
 is_user_switch_enabled (GSWindow *window)
 {
-  	return window->priv->user_switch_enabled;
+        return window->priv->user_switch_enabled;
 }
 
 static gboolean
@@ -1422,13 +1532,13 @@ popup_dialog_idle (GSWindow *window)
                 g_string_append_printf (command, " --logout-command='%s'", window->priv->logout_command);
         }
 
-	if (window->priv->away_message) {
-		char *quoted;
+        if (window->priv->away_message) {
+                char *quoted;
 
-		quoted = g_shell_quote (window->priv->away_message);
-		g_string_append_printf (command, " --away-message=%s", quoted);
-		g_free (quoted);
-	}
+                quoted = g_shell_quote (window->priv->away_message);
+                g_string_append_printf (command, " --away-message=%s", quoted);
+                g_free (quoted);
+        }
 
         if (is_user_switch_enabled (window)) {
                 command = g_string_append (command, " --enable-switch");
@@ -1438,7 +1548,10 @@ popup_dialog_idle (GSWindow *window)
                 command = g_string_append (command, " --verbose");
         }
 
-        gs_window_clear (window);
+        gtk_widget_hide (window->priv->drawing_area);
+
+        gs_window_clear_to_background_pixmap (window);
+
         set_invisible_cursor (GTK_WIDGET (window)->window, FALSE);
 
         result = spawn_on_window (window,
@@ -1496,12 +1609,12 @@ gs_window_cancel_unlock_request (GSWindow  *window)
 	 */
         g_return_if_fail (GS_IS_WINDOW (window));
 
-	if (window->priv->lock_socket == NULL) {
-		return;
+        if (window->priv->lock_socket == NULL) {
+                return;
         }
 
-	if (window->priv->lock_pid > 0) {
-		kill (window->priv->lock_pid, SIGTERM);
+        if (window->priv->lock_pid > 0) {
+                kill (window->priv->lock_pid, SIGTERM);
         }
 }
 
@@ -1710,9 +1823,9 @@ gs_window_get_property (GObject    *object,
         case PROP_LOGOUT_COMMAND:
                 g_value_set_string (value, self->priv->logout_command);
                 break;
-	case PROP_AWAY_MESSAGE: 
-		g_value_set_string (value, self->priv->away_message);
-		break;
+        case PROP_AWAY_MESSAGE:
+                g_value_set_string (value, self->priv->away_message);
+                break;
         case PROP_LOGOUT_TIMEOUT:
                 g_value_set_long (value, self->priv->logout_timeout);
                 break;
@@ -1859,7 +1972,7 @@ gs_window_real_size_request (GtkWidget      *widget,
 {
         GSWindow      *window;
         GtkBin        *bin;
-	GdkRectangle   old_geometry;
+        GdkRectangle   old_geometry;
         int            position_changed = FALSE;
         int            size_changed = FALSE;
 
@@ -1870,7 +1983,7 @@ gs_window_real_size_request (GtkWidget      *widget,
                 gtk_widget_size_request (bin->child, requisition);
         }
 
-	old_geometry = window->priv->geometry;
+        old_geometry = window->priv->geometry;
 
         update_geometry (window);
 
@@ -2121,11 +2234,16 @@ gs_window_init (GSWindow *window)
                                | GDK_VISIBILITY_NOTIFY_MASK
                                | GDK_ENTER_NOTIFY_MASK
                                | GDK_LEAVE_NOTIFY_MASK);
-        force_no_pixmap_background (GTK_WIDGET (window));
 
         window->priv->vbox = gtk_vbox_new (FALSE, 12);
         gtk_widget_show (window->priv->vbox);
         gtk_container_add (GTK_CONTAINER (window), window->priv->vbox);
+
+        window->priv->drawing_area = gtk_drawing_area_new ();
+        gtk_widget_show (window->priv->drawing_area);
+        gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->drawing_area, TRUE, TRUE, 0);
+
+        force_no_pixmap_background (window->priv->drawing_area);
 }
 
 static void
